import logging
import boto3

from typing import List

logr = logging.getLogger(__name__)


class ApiGatewayDescriber:
    """
    Resource description type class the describes api gateways.
    More on API Gateway:
    https://aws.amazon.com/api-gateway/
    """
    def __init__(self, gateway_name: str):
        """
        Constructor.

        :param gateway_name: API Gateway name.
        """
        self.gateway_name = gateway_name

    def describe(self) -> str:
        """
        Returns a resource id for specified api gateway.

        :return: Resource id.
        """
        logr.info(f'Getting api gateway rest api id by name {self.gateway_name}...')

        api_id = None
        api_names: List[str] = []
        for api in dict(boto3.client('apigateway').get_rest_apis().items())['items']:
            api_names.append(api['name'])

            if api['name'] == self.gateway_name:
                api_id = api['id']

        logr.info(f'Found {len(api_names)} api gateways: {api_names}.')

        assert api_id, 'Rest api id could not be found.'

        return api_id



from aws_infrastructure_sdk.s3.s3_abstract_action import AbstractS3Action


class S3BucketDeleter(AbstractS3Action):
    """
    Class that deletes S3 buckets.
    """
    def __init__(self):
        """
        Constructor.
        """
        super().__init__()

    def delete_with_prefix(self, prefix: str) -> None:
        """
        Deletes buckets with specific prefixes.

        :param prefix: Prefix for bucket names.

        :return: No return.
        """
        buckets = self.s3_client.list_buckets()

        for bucket in buckets['Buckets']:
            name = bucket['Name']  # type: str

            if name.startswith(prefix):
                s3_bucket = self.s3_resource.Bucket(name)
                s3_bucket.objects.all().delete()
                s3_bucket.delete()



import logging

from aws_infrastructure_sdk.bash.command import BashCommand

logr = logging.getLogger(__name__)


class ZappaDeployer:
    """
    Deploys zappa project.
    More on zappa tool:
    https://www.zappa.io/
    """
    def __init__(self, project_path: str, stage: str):
        """
        Constructor.

        :param project_path: Path to a zappa project.
        :param stage: Stage of the project (currently supported dev or prod).
        """
        assert stage in ['dev', 'prod'], 'Unsupported stage.'

        self.stage = stage
        self.project_path = project_path

    def deploy(self) -> bool:
        """
        Initiate zappa deployment.

        :return: A boolean value indicating whether deployment was successful.
        """
        deploy_command = (
            "cd {} "
            # We activate a virtual environment because zappa uses the active python
            # environment to package the dependencies.
            "&& . tmpenv/bin/activate "
            # We first call zappa update because it is most likely the 
            # project is deployed not the first time.
            "&& ( zappa update {} || zappa deploy {} )"
        ).format(self.project_path, self.stage, self.stage)

        try:
            logr.info('Deploying...')
            success = BashCommand(deploy_command).run()
            assert success, 'Deployment failed.'
            logr.info('Deployment successful.')
            return True
        except AssertionError as ex:
            logr.error(repr(ex))
            return False



import os
import shutil
import logging

from typing import Dict, Any
from aws_infrastructure_sdk.bash.command import BashCommand
from aws_infrastructure_sdk.cloud_formation.api_gateway_describer import ApiGatewayDescriber
from aws_infrastructure_sdk.git.git_clone import GitClone
from aws_infrastructure_sdk.lambdas.env_updater import LambdaEnvUpdater
from aws_infrastructure_sdk.s3.s3_bucket_deleter import S3BucketDeleter
from aws_infrastructure_sdk.zappa.zappa_env_updater import ZappaEnvUpdater
from aws_infrastructure_sdk.zappa.zappa_config import ZappaConfig

logr = logging.getLogger(__name__)


class ZappaDjangoDeploy:
    """
    Zappa - Django project deployment class.
    """
    DEFAULT_CLONE_PATH = '/tmp/zappa-django-tmp-project'

    def __init__(
            self,
            project_git_url: str,
            project_environment_vars: Dict[str, Any],
            zappa_environment_vars: Dict[str, Any],
            stage: str,
            deployed_project_name: str,
            ssh_file_path: str,
            aws_region: str
    ) -> None:
        """
        Constructor.

        :param project_git_url: Project under deployment git repository url.
        :param project_environment_vars: Project under deployment environment variables.
        :param zappa_environment_vars: Project's under deployment zappa environment variables.
        :param stage: Deployment stage (currently supported dev or prod).
        :param deployed_project_name: Name of the project in AWS infrastructure when it is deployed.
        :param ssh_file_path: Path to ssh file for git clone action without a password.
        :param aws_region: Region where the AWS infrastructure is.
        """
        assert stage in ['dev', 'prod'], 'Unsupported stage.'

        self.aws_region = aws_region
        self.ssh_file_path = ssh_file_path
        self.project_git_url = project_git_url
        self.project_environment = project_environment_vars
        self.zappa_environment = zappa_environment_vars
        self.stage = stage
        self.deployed_project_name = deployed_project_name
        self.deployed_project_name_with_stage = self.deployed_project_name + '-' + self.stage

        # We make sure in initial stage it allows all hosts.
        # Used so zappa deploy would not fail.
        self.project_environment['ALLOWED_HOST'] = '*'

    def deploy(self) -> None:
        """
        Initiates project deployment.

        :return: No return.
        """
        logr.info(
            f'Initiating project deployment. Context:'
            f'\nRegion: {self.aws_region},'
            f'\nSSH file path: {self.ssh_file_path},'
            f'\nProject git url: {self.project_git_url},'
            f'\nProject environment: {self.project_environment},'
            f'\nZappa environment: {self.zappa_environment},'
            f'\nStage: {self.stage},'
            f'\nDeployed project name: {self.deployed_project_name},'
            f'\nDeployed project name with stage: {self.deployed_project_name_with_stage}.'
        )

        # Clean previous failed builds.
        self.__clean()

        logr.info(f'Downloading {self.deployed_project_name_with_stage} project...')
        GitClone(self.ssh_file_path).clone(self.project_git_url, self.DEFAULT_CLONE_PATH)

        logr.info(f'Updating {self.deployed_project_name_with_stage} zappa settings...')
        ZappaEnvUpdater(self.DEFAULT_CLONE_PATH).update(self.project_environment, self.zappa_environment)

        # Install all dependencies.
        self.__install()
        # Run tests and assert whether it can be deployed to servers.
        self.__test()
        # Deploy to servers.
        self.__deploy()
        # Clean build dirs.
        self.__clean()

        # Zappa deploys lambda function and creates an api gateway.
        # The gateway host must be added to django security hosts.
        api_gateway_id = ApiGatewayDescriber(self.deployed_project_name_with_stage).describe()

        logr.info('Updating lambda environment...')
        self.project_environment['ALLOWED_HOST'] = '{}.execute-api.{}.amazonaws.com'.format(api_gateway_id, self.aws_region)
        LambdaEnvUpdater(self.deployed_project_name_with_stage).update(self.project_environment)

        logr.info('Deployment fully configured and successful!')

    def __clean(self):
        """
        Deletes build leftovers.

        :return: No return.
        """
        # Delete build leftovers
        logr.info(f'Deleting build leftovers for {self.deployed_project_name_with_stage}...')
        try:
            shutil.rmtree(self.DEFAULT_CLONE_PATH)
        except FileNotFoundError:
            pass

        # Delete upload leftovers
        logr.info(f'Deleting zappa buckets for {self.deployed_project_name_with_stage}...')
        # When deploying with zappa a lot of buckets are being created. Delete them.
        S3BucketDeleter().delete_with_prefix('zappa')

    def __test(self):
        """
        Runs projects tests.

        :return: No return.
        """
        # Create dotenv file for testing .
        self.__create_dot_env()

        test_command = (
            "cd {} "
            # Source the virtual env.
            "&& . tmpenv/bin/activate "
            # Run django tests.
            "&& echo "
            "&& python manage.py test --noinput".format(self.DEFAULT_CLONE_PATH)
        )

        logr.info(f'Running tests for {self.deployed_project_name_with_stage}...')
        success = BashCommand(test_command).run()
        assert success, 'Tests failed!'
        logr.info(f'Tests succeeded for {self.deployed_project_name_with_stage}!')

        # After testing delete the file.
        self.__delete_dot_env()

    def __create_dot_env(self):
        """
        Creates dot-env file for storing environment.

        :return: No return.
        """
        with open(self.DEFAULT_CLONE_PATH + '/.env', 'w') as env:
            for key, value in self.project_environment.items():
                env.write(key + '=' + value + '\n')

    def __delete_dot_env(self):
        """
        Deletes dot-env file.

        :return: No return.
        """
        os.remove(self.DEFAULT_CLONE_PATH + '/.env')

    def __install(self):
        """
        Installs projects with it's dependencies.

        :return: No return.
        """
        install_command = (
            "cd {} "
            "&& virtualenv tmpenv --python=python3"
            "&& . tmpenv/bin/activate"
            # Every nj project must contain install.sh script at the root directory
            # which will fully install an configure the system to successfully run the project.
            "&& ./install.sh python {}"
            # Install zappa to enable deployment.
            "&& pip install zappa=={}".format(
                self.DEFAULT_CLONE_PATH,
                self.stage,
                ZappaConfig.VERSION
            ))

        logr.info(f'Installing {self.deployed_project_name_with_stage} project...')
        success = BashCommand(install_command).run()
        assert success, 'Installation failed!'
        logr.info('Installation succeeded!')

    def __deploy(self):
        """
        Deploys project.

        :return: None.
        """
        deploy_command = (
            "cd {} "
            # We activate a virtual environment because zappa uses the active python
            # environment to package the dependencies.
            "&& . tmpenv/bin/activate "
            "&& ( zappa update {} || zappa deploy {} )"
        ).format(self.DEFAULT_CLONE_PATH, self.stage, self.stage)

        logr.info(f'Deploying {self.deployed_project_name_with_stage}...')
        success = BashCommand(deploy_command).run()
        assert success, 'Deployment failed.'
        logr.info(f'Deployment for {self.deployed_project_name_with_stage} was successfull!')



import logging
import subprocess

from aws_infrastructure_sdk.utils.dir_utils import DirUtils

logr = logging.getLogger(__name__)


class GitClone:
    """
    Wrapper class for git clone action.
    """
    def __init__(self, path_to_ssh_file: str):
        """
        Constructor.

        :param path_to_ssh_file: Path to an ssh file which allows to fetch protected repositories without password.
        More on git ssh files:
        https://git-scm.com/book/en/v2/Git-on-the-Server-Generating-Your-SSH-Public-Key
        """
        self.path_to_ssh_file = path_to_ssh_file

    def clone(self, git_url: str, download_path: str) -> None:
        """
        Clones a git repository.

        :param git_url: Repository url formatted for ssh use: e.g. git@bitbucket.org:person/repository.git
        :param download_path: Target path of where to clone.

        :return: No return.
        """
        DirUtils.create_if_not_exists(download_path)

        prcs = subprocess.Popen("bash", shell=True, stdin=subprocess.PIPE, stdout=None, stderr=subprocess.PIPE)
        stdout, stderr = prcs.communicate("ssh-agent $(ssh-add {}; git clone {} {})".format(
            self.path_to_ssh_file,
            git_url,
            download_path
        ).encode())

        if stdout:
            logr.info(stdout)

        if stderr:
            logr.error(stderr)



import logging
import subprocess

logr = logging.getLogger(__name__)


class BashCommand:
    """
    Wrapper class for bash command execution.
    """
    def __init__(self, command: str):
        """
        Constructor.

        :param command: Command to execute in bash shell.
        """
        self.command = command

    def run(self) -> bool:
        """
        Execute the command.

        :return: Boolean value indicating whether command executed successfully.
        """
        process = subprocess.Popen(
            self.command,
            shell=True,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )

        output, err = process.communicate()

        if process.returncode != 0:
            logr.info(output.decode())
            logr.error(err.decode())

        return process.returncode == 0



from botocore.exceptions import ClientError
from aws_infrastructure_sdk.cloud_formation.stack.abstract_stack_action import AbstractStackAction


class ResourceDescriber(AbstractStackAction):
    """
    Resources created with cloud-formation stack describer class.
    """
    def __init__(self, cf_stack_name: str):
        """
        Constructor.

        :param cf_stack_name: The name of cloud-formation stack.
        """
        super().__init__(cf_stack_name)

    def describe(self, logical_resource_id: str):
        """
        Returns physical resource id by a logical resource id.

        :param logical_resource_id: Id of the stack resource.

        :return: Physical resource id by a logical resource id.
        """
        try:
            return self.cf_client.describe_stack_resource(
                StackName=self.cf_stack_name,
                LogicalResourceId=logical_resource_id
            )['StackResourceDetail']['PhysicalResourceId']
        except ClientError:
            self.get_logger().warning('Resource with logical id {} does not exist'.format(logical_resource_id))
            raise



from typing import Dict, Any


class Log:
    def __init__(self):
        pass

    @staticmethod
    def default_logging_formatters() -> Dict[str, Any]:
        return {
            'standard': {
                'format': '[%(levelname)-8s] '
                          'time: "%(asctime)s" '
                          'pid: "%(process)d" '
                          'tid: "%(thread)d" '
                          'module: "%(module)s" '
                          'message: "%(message)s"'
            },
            'console': {
                'format': '[%(levelname)-8s] '
                          'module: "%(module)s" '
                          'message: "%(message)s"'
            }
        }

    @staticmethod
    def default_logging_handlers() -> Dict[str, Any]:
        return {
            'null': {
                'level': 'DEBUG',
                'class': 'logging.NullHandler',
            },
            'console': {
                'class': 'logging.StreamHandler',
                'formatter': 'console'
            },
        }

    @staticmethod
    def default_loggers() -> Dict[str, Any]:
        return {
            '': {
                'handlers': ['console'],
                'level': 'INFO',
                'propagate': True
            },
        }

    @staticmethod
    def default_logging_config() -> Dict[str, Any]:
        return {
            'version': 1,
            'disable_existing_loggers': False,
            'formatters': Log.default_logging_formatters(),
            'handlers': Log.default_logging_handlers(),
            'loggers': Log.default_loggers()
        }



from aws_infrastructure_sdk.cloud_formation.stack.abstract_stack_action import AbstractStackAction


class StackDeleter(AbstractStackAction):
    """
    Cloud-formation stack deleter class.
    """
    def __init__(self, cf_stack_name: str):
        """
        Constructor.

        :param cf_stack_name: The name of cloud-formation stack.
        """
        super().__init__(cf_stack_name)

    def delete(self):
        """
        Deletes the cloud-formation stack.

        :return: No return.
        """
        self.get_logger().info('Deleting stack {}...'.format(self.cf_stack_name))
        self.cf_client.delete_stack(StackName=self.cf_stack_name)



from aws_infrastructure_sdk.s3.s3_abstract_action import AbstractS3Action


class S3BucketCreator(AbstractS3Action):
    def __init__(self, bucket_name: str, region: str):
        """
        Constructor.

        :param bucket_name: The name of S3 bucket.
        :param region: Region where a bucket exists.
        """
        super().__init__()

        self.region = region
        self.bucket_name = bucket_name

    def create(self) -> None:
        """
        Creates an S3 bucket if it does not exist.

        :return: No return.
        """
        exists = self.bucket_name in [bucket['Name'] for bucket in self.s3_client.list_buckets()['Buckets']]

        # If bucket does not exist - create it.
        if not exists:
            self.get_logger().info('Bucket does not exist. Creating {}...'.format(self.bucket_name))

            self.s3_client.create_bucket(
                Bucket=self.bucket_name,
                ACL='private',
                CreateBucketConfiguration={
                    'LocationConstraint': self.region
                }
            )
        else:
            self.get_logger().info('Bucket {} already exists.'.format(self.bucket_name))



import re
import subprocess

from typing import Optional


class GitVersionResolver:
    """
    Git tags as versions resolver class.
    """
    def __init__(self, repository_url: str):
        """
        Constructor.

        :param repository_url: Repository url formatted for ssh use: e.g. git@bitbucket.org:person/repository.git
        """
        self.git_repository = repository_url

    def resolve(self, version_to_resolve: str, dev: bool = True) -> Optional[str]:
        """
        Resolves a git tagged version. E.g. If version to resolve is 2.*.*, function fetches all
        tags from a provided git repository and assigns latest tags to wildcards in version to resolve string.

        :return: Resolved version string or None if resolve failed.
        """
        # Split version to resolve string into major, minor and patch versions
        major, minor, patch = version_to_resolve.split('.')

        # Regex to match only those versions that contain only numbers, digits and a string 'dev-'
        regex = re.compile(r'^[dev\-0-9.]*$')

        # Fetch all tags from a git repository
        output = subprocess.check_output(
            ['git', 'ls-remote', '--tags', 'ssh://{}'.format(self.git_repository)])
        # Split and filter out standard git api output and leave only tag strings
        output = [line.split('\t')[1].split('/')[2] for line in output.decode().splitlines()]
        # Filter our tags that do not match version structure (e.g. dev-2.0.0 or 1.5.0)
        output = list(filter(regex.search, output))

        # Separate development versions
        dev_versions = [version.replace('dev-', '') for version in output if version.__contains__('dev')]
        # Separate production versions
        prod_versions = [version for version in output if not version.__contains__('dev')]

        # Set versions list to work with depending on environment (dev or prod)
        versions = dev_versions if dev else prod_versions

        # Split version strings  into a list of 3 integers: major, minor and patch versions
        versions = [[int(version) for version in version_string.split('.')] for version_string in versions]
        # Sort in reverse versions so the latest version would be at index 0
        versions.sort(key=lambda version: (version[0], version[1], version[2]), reverse=True)

        # Create a function that resolves each subversion independently
        def resolve_subversion(versions, index, subversion):
            if subversion != '*':
                versions = [vrs for vrs in versions if vrs[index] == int(subversion)]
            return versions, versions[0][index]

        try:
            # Resolve major version first
            versions, resolved_major = resolve_subversion(versions, 0, major)
            # Resolve minor version second
            versions, resolved_minor = resolve_subversion(versions, 1, minor)
            # Resolve patch version last
            versions, resolved_patch = resolve_subversion(versions, 2, patch)
        except IndexError:
            print('Failed to resolve version for: {}'.format(version_to_resolve))
            return None

        # Return a resolved version string which corresponds to input version to resolve
        return str(resolved_major) + '.' + str(resolved_minor) + '.' + str(resolved_patch)



import boto3

from typing import List, Optional


class AvailabilityZones:
    """
    Class that shows current availability zones.
    More on availability zones:
    https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-regions-availability-zones.html
    """
    def __init__(self, region: str, expected_az_count: Optional[int] = None):
        """
        Constructor.

        :param region: Name of the AWS Region.
        :param expected_az_count: Expected availability zones count.
        """
        self.region = region
        self.expected_az_count = expected_az_count

    def get(self) -> List[str]:
        """
        Returns a list of availability zones.

        :return: List of availability zones
        """
        ec2 = boto3.client('ec2')

        # Retrieves all regions/endpoints that work with EC2
        zones = ec2.describe_availability_zones()['AvailabilityZones']
        zones = [zone['ZoneName'] for zone in zones if zone['RegionName'] == self.region]

        # Ensure consistency
        zones = sorted(zones)

        if self.expected_az_count is not None:
            # Ensure that the amount of availability zones does not change (because of a global disaster for e.g.)
            # Otherwise manual actions will be required
            assert len(zones) == self.expected_az_count

        return zones



import logging
import boto3

from typing import Optional
from abc import ABC


class AbstractStackAction(ABC):
    """
    Abstract class indicating an abstract cloud-formation stack action.
    More on cloud-formation:
    https://aws.amazon.com/cloudformation/
    """
    def __init__(self, cf_stack_name: str):
        """
        Constructor.

        :param cf_stack_name: The name of cloud-formation stack.
        """
        self.cf_stack_name = cf_stack_name
        self.cf_client = boto3.client('cloudformation')

    @staticmethod
    def get_logger(name: Optional[str] = None):
        """
        Creates logger instance.

        :param name: The name of the logger.

        :return: Logger instance.
        """
        return logging.getLogger(name or __name__)



import logging
import boto3

from typing import Optional
from abc import ABC


class AbstractS3Action(ABC):
    """
    Abstract class which specifies a AWS S3 action.
    More on S3:
    https://aws.amazon.com/s3/
    """
    def __init__(self):
        """
        Constructor.
        """
        self.s3_client = boto3.client('s3')
        self.s3_resource = boto3.resource('s3')

    @staticmethod
    def get_logger(name: Optional[str] = None):
        """
        Returns a logger instance.

        :param name: Logger name

        :return: Logger instance.
        """
        return logging.getLogger(name or __name__)



from botocore.exceptions import ClientError
from aws_infrastructure_sdk.cloud_formation.stack.abstract_stack_action import AbstractStackAction


class StackStatus(AbstractStackAction):
    """
    Cloud-formation stack status checker class.
    """
    def __init__(self, cf_stack_name: str):
        """
        Constructor.

        :param cf_stack_name: The name of cloud-formation stack.
        """
        super().__init__(cf_stack_name)

    def status(self) -> str:
        """
        Checks what is the current status of the default stack.

        :return: Stack status constant.
        """
        self.get_logger().info('Getting stack {} status...'.format(self.cf_stack_name))

        try:
            response = self.cf_client.describe_stacks(StackName=self.cf_stack_name)
            return response['Stacks'][0]['StackStatus']
        except ClientError as ex:
            self.get_logger().error('Can not check stack status. Reason: {}'.format(repr(ex)))
            raise



import boto3
import logging

from typing import Dict, Any

logr = logging.getLogger(__name__)


class LambdaEnvUpdater:
    """
    AWS Lambda environment updater.
    More on lambda:
    https://aws.amazon.com/lambda/
    """
    def __init__(self, lambda_name: str):
        """
        Constructor.

        :param lambda_name: The name of the AWS Lambda function.
        """
        self.lambda_name = lambda_name

    def update(self, env: Dict[str, Any]):
        """
        Updates lambda function's environment.

        :param env: Environment with which a lambda should be updated.

        :return: No return.
        """
        logr.info('Updating lambda environment...')

        boto3.client('lambda').update_function_configuration(
            FunctionName=self.lambda_name,
            Environment={
                'Variables': env,
            }
        )



from typing import Any, Dict, List
from aws_infrastructure_sdk.cloud_formation.stack.abstract_stack_action import AbstractStackAction
from aws_infrastructure_sdk.s3.s3_bucket_creator import S3BucketCreator
from aws_infrastructure_sdk.s3.uploader import Uploader


class StackDeployer(AbstractStackAction):
    """
    Cloud-formation stack deployment class.
    """
    def __init__(self, cf_stack_name: str, region: str):
        """
        Constructor.

        :param cf_stack_name: The name of cloud-formation stack.
        """
        super().__init__(cf_stack_name)

        self.region = region

    def deploy(self, cf_bucket_name: str, template: str, parameters: List[Dict[str, Any]]) -> None:
        """
        Updates or creates cloud-formation stack.

        :param cf_bucket_name: The name of the S3 bucket where a cloud-formation template should be uploaded.
        :param template: Generated cloud-formation template.
        :param parameters: Parameters for the template.

        :return: No return.
        """
        self.get_logger().info('Deploying stack {}...'.format(self.cf_stack_name))
        self.get_logger().info('Uploading cloudformation template to S3...')

        S3BucketCreator(cf_bucket_name, self.region).create()
        s3_url = Uploader(cf_bucket_name).upload_bytes(template.encode('utf-8'))

        kwargs = {
            'StackName': self.cf_stack_name,
            'TemplateURL': s3_url,
            'Capabilities': [
                'CAPABILITY_IAM',
            ],
            'Parameters': parameters
        }

        try:
            response = self.cf_client.create_stack(**kwargs)
        except self.cf_client.exceptions.AlreadyExistsException:
            response = self.cf_client.update_stack(**kwargs)

        self.get_logger().info('Done! Stack response: {}'.format(response))



import json
import logging
import os

from typing import Any, Dict

logr = logging.getLogger(__name__)


class ZappaEnvUpdater:
    """
    Zappa project environment file updater class.
    """
    def __init__(self, project_path: str):
        """
        Constructor.

        :param project_path: Path to a zappa project.
        """
        self.project_path = project_path

    def update(self, project_environment: Dict[str, Any], zappa_environment: Dict[str, Any]):
        """
        Updates zappa_settings.json file for a zappa project.

        :param project_environment: Environment variables for the project.
        :param zappa_environment:  Environment variables for zappa.
        """
        logr.info('Updating settings...')

        with open(os.path.join(self.project_path, 'zappa_settings.json'), 'r+') as settings:
            settings_dict = json.loads(settings.read())

            # Cycle through every stage (dev, prod, etc.)
            for settings_stage_key, stage_dict in settings_dict.items():
                stage_environment = settings_dict[settings_stage_key]

                # Update project env variables
                stage_environment['environment_variables'] = project_environment

                # Update zappa settings
                for key, value in zappa_environment.items():
                    stage_environment[key] = value

            settings_string = json.dumps(settings_dict)
            settings.seek(0)
            settings.write(settings_string)



import datetime
import ntpath

from io import BytesIO
from aws_infrastructure_sdk.s3.s3_abstract_action import AbstractS3Action


class Uploader(AbstractS3Action):
    def __init__(self, bucket_name: str):
        super().__init__()
        self.bucket_name = bucket_name

    def upload_bytes(self, bytes_object: bytes):
        """
        Uploads bytes object to a specified bucket and returns a pre-signed url.
        """
        self.get_logger().info('Uploading bytes object to S3...')

        bucket = self.s3_resource.Bucket(self.bucket_name)

        output = BytesIO()
        output.write(bytes_object)
        output.seek(0)

        s3_obj_name = str(datetime.datetime.now())
        bucket.upload_fileobj(output, s3_obj_name)

        s3_url = self.s3_client.generate_presigned_url('get_object', Params={
            'Bucket': self.bucket_name,
            'Key': s3_obj_name
        })

        self.get_logger().info('Finished uploading bytes object to S3.')

        return s3_url

    def upload_file(self, path_to_file: str):
        """
        Uploads a file in a given bucket and returns a pre-signed url.
        """
        self.s3_client.upload_file(path_to_file, self.bucket_name, ntpath.basename(path_to_file))

        return self.s3_client.generate_presigned_url(
            'get_object',
            Params={'Bucket': self.bucket_name, 'Key': ntpath.basename(path_to_file)}
        )



import os
import subprocess
import logging

from typing import Optional
from aws_infrastructure_sdk.lambdas.deployment import lambda_deployment_dir
from aws_infrastructure_sdk.s3.s3_bucket_creator import S3BucketCreator

logr = logging.getLogger(__name__)


class DeploymentPackage:
    def __init__(
            self,
            project_src_path: str,
            lambda_name: str,
            s3_upload_bucket: str,
            s3_bucket_region: str,
            aws_profile: str,
            environment: Optional[str] = None,
            refresh_lambda: bool = False
    ) -> None:
        self.__refresh_lambda = refresh_lambda
        self.__aws_profile = aws_profile
        self.__environment = environment or ''
        self.__s3_upload_bucket = s3_upload_bucket
        self.__s3_bucket_region = s3_bucket_region
        self.__project_src_path = project_src_path
        self.__lambda_name = lambda_name
        self.__dir = lambda_deployment_dir

        self.__build_command = [
            os.path.join(self.__dir, 'build.sh'),
            self.__environment,
            self.__project_src_path
        ]

        upload_options = ['-s']
        if self.__refresh_lambda:
            upload_options.append('-l')

        self.__upload_command = [
            os.path.join(self.__dir, 'upload.sh'),
            self.__lambda_name,
            self.__s3_upload_bucket,
            self.__aws_profile,
            self.__s3_bucket_region
        ]

        self.__upload_command.extend(upload_options)

    def deploy(self):
        # Actually we do not have to check what type of environment is passed. The script will assert this
        # part and throw an exception if the value is not valid. However this is purely optimization part
        # which would save quite some time if an incorrect environment is provided. If installation
        # scripts tend to change and support more environments - delete or update the line below.
        assert self.__environment in ['dev', 'prod', 'none'], 'Unsuppored env!'

        # Ensure that provided bucket exists.
        S3BucketCreator(self.__s3_upload_bucket, self.__s3_bucket_region).create()

        try:
            logr.info('Installing...')
            output = subprocess.check_output(self.__build_command, stderr=subprocess.STDOUT)
            logr.info(output.decode())

            logr.info('Uploading...')
            output = subprocess.check_output(self.__upload_command, stderr=subprocess.STDOUT)
            logr.info(output.decode())
        except subprocess.CalledProcessError as ex:
            logr.error(ex.output.decode())
            raise



from aws_infrastructure_sdk.s3.s3_abstract_action import AbstractS3Action


class S3BucketChecker(AbstractS3Action):
    """
    Checker class that does various checks against S3 buckets.
    """
    def is_empty(self, bucket_name: str):
        """
        Checks for contents in a given S3 bucket.

        :param bucket_name: The name of S3 bucket.

        :return: True if bucket empty, False otherwise.
        """
        resp = self.s3_client.list_objects_v2(Bucket=bucket_name)
        return len(resp['Contents']) == 0



from setuptools import setup, find_packages

setup(
    name='aws_infrastructure_sdk',
    version='1.1.1',
    license='GNU GENERAL PUBLIC LICENSE Version 3',
    packages=find_packages(),
    description='SDK that helps to build AWS - CloudFormation infrastructure projects.',
    include_package_data=True,
    install_requires=[
        'boto3',
        'botocore'
    ],
    author='Laimonas Sutkus',
    author_email='laimonas.sutkus@gmail.com',
    keywords='AWS SDK CloudFormation Zappa Infrastructure Cloud',
    url='https://github.com/laimonassutkus/AwsCfSdk',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent',
    ],
)



import os


class DirUtils:
    """
    Utility class used for various actions with directories.
    """
    @staticmethod
    def exists(path: str) -> bool:
        """
        Tells whether the directory under given path exists.

        :param path: Path to a directory to check.

        :return: Boolean value indicating whether directory exists.
        """
        return os.path.exists(path)

    @staticmethod
    def create_if_not_exists(path: str) -> None:
        """
        Creates a directory if it does not exist.

        :param path: Path where a directory should be created.

        :return: No return.
        """
        if not DirUtils.exists(path):
            os.makedirs(path)
